﻿// Windows (VisualStudio)
//  cl /LD /Fereverse.wfm reverse.c
//
// Linux (GCC)
//  gcc -O -shared -fPIC -o reverse.lfm reverse.c

///////////////////////////////////////////////////////////////////////////////
// ヘッダーファイル
#include <string.h>
#include <wchar.h>

///////////////////////////////////////////////////////////////////////////////
// Windows用エクスポート宣言
#ifdef _WIN32
#ifndef _WIN64
#error 64bitバイナリーとしてビルドしてください
#endif
#define EXPORT __declspec(dllexport)
#else
#define EXPORT
#endif

///////////////////////////////////////////////////////////////////////////////
// 定数(タイプ)
#define	TYPE_8      0x0001
#define	TYPE_16     0x0002
#define	TYPE_32     0x0004
#define	TYPE_24     0x0008
#define	TYPE_48     0x0010
#define	TYPE_96     0x0020
#define	TYPE_192    0x0040
#define	TYPE_UI     0x0100

///////////////////////////////////////////////////////////////////////////////
// 定数
#define	ON          1
#define	OFF         0

///////////////////////////////////////////////////////////////////////////////
// プラグイン関数
//=============================================================================
// 必須
//=============================================================================
const char EXPORT *em_name(const char *lang)
{
	// メニュー項目・フィルターリスト用の名前文字列
	// lang は "jp"(日本語) か "en"(英語) が指定される

	// 名前を返す(例)
	return strcmp(lang, "jp") ? "Reverse" : "反転";
}

//=============================================================================
// 必須
//=============================================================================
unsigned int EXPORT em_init(void(*redraw_pixel)(void), void(*redraw_view)(void), void(*close_ui)(void* instance), void(*set_videoframe)(size_t frame), void(*msg_func)(void), double(*get_scale)(int *axis_y, const wchar_t **unit))
{
	// 対応しているフォーマットと機能を返す
	return TYPE_24;	// 24bitカラーとして処理する(元画像が48bitカラー等の場合は24bitカラーに劣化させてしまう)
}

//=============================================================================
// 必須
//=============================================================================
void EXPORT *em_create(void)
{
	return (void*)1;	// ローカル領域不要なので適当な値(0以外)を返す
}

//=============================================================================
// 必須
//=============================================================================
void EXPORT em_delete(void *efmh)
{
}

///////////////////////////////////////////////////////////////////////////////
// プラグイン関数(個別インスタンス毎の処理)
//=============================================================================
// 
//=============================================================================
int EXPORT on_filter(void *efmh, void *img, size_t w, size_t h, unsigned short bpp, size_t frame, size_t sx, size_t sy, size_t ex, size_t ey, int connect, int bg)
{
	// 画像の加工や計測等の実処理部分
	// img, w, h, bpp は画像バッファ情報
	// frame 0～ 動画・連番画像の場合は 0,1,…とフレーム番号が入る
	// sx, sy, ex, ey は処理領域
	// connectは 4or8 近傍
	// bgは2値画像処理時の背景色 0.黒 !0.白
	size_t x, y;
	unsigned char *p;

	// 指定矩形の輝度(RGB個別)を反転させる
	for (y=sy; y<=ey; y++) {
		for (x=sx; x<=ex; x++) {
			p = (unsigned char*)img + (y * w * 3) + (x * 3);
			p[0] = ~p[0];
			p[1] = ~p[1];
			p[2] = ~p[2];
		}
	}

	// 画像を変更してるならON、変更していないならOFFを返す
	return ON;
}

